import json

try:
    import bpy
    import mathutils
    from . import scene
    from .. import utils
    from .. import config
    from ..tools import bouncecurve as bc
    from ..bouncecurve import bcutils
except ModuleNotFoundError as e:
    import scene
    import utils
    import config
    import bcutils
    from tools import bouncecurve as bc


class RectifyOffsetOperator(bpy.types.Operator):
    bl_idname = "object.bc_rectify_offset"
    bl_label = "偏移校正"
    bl_description = "调整好曲线后再来校正，校正后再添加其他关键帧"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        data = context.scene.bc_score_data
        try:
            j = json.loads(data)
        except:
            self.report({'ERROR'}, '乐谱数据丢失，请重新生成乐谱')
            return {'FINISHED'}
        obj_bounce_curve = context.scene.bc_obj_bounce
        if not obj_bounce_curve:
            self.report({'INFO'}, '请选择一个弹跳物体')

        obj_scales = context.scene.bc_obj_svg_score.scale
        if obj_scales[0] != obj_scales[2]:
            self.report({'INFO'}, '请保证乐谱X/Y轴以等比例缩放')
            return {'FINISHED'}
        parts = j.get('parts', [])
        obj_bounce_ind = 0
        for part in parts:
            times = [t.get('onset_time') for t in part.get('times', [])]
            if len(times) < 1:
                self.report({'INFO'}, '乐谱数据异常1')
                continue
            note_paths = part.get('note_paths', [])

            for obj_ind, item in enumerate(context.scene.bc_real_obj_bounce[obj_bounce_ind:]):
                if obj_ind >= len(note_paths):
                    break
                path = note_paths[obj_ind]
                ani_times = [t for i, t in enumerate(times) if path[i] is not None]
                obj = item.obj
                location = obj.location.copy()
                curve = obj.modifiers['curve'].object
                if curve is None:
                    self.report({'INFO'}, '弹跳物体{}未设置曲线'.format(obj.name))
                    return {'FINISHED'}

                floor_dists = bcutils.calculate_curve_section_length(curve)
                if len(floor_dists) != len(ani_times):
                    self.report({'INFO'}, '弹跳物体{}对应弹跳曲线变更不符合规则'.format(obj.name))
                    return {'FINISHED'}
                delta = location[0] - floor_dists[0]
                obj.animation_data_clear()
                for i, dist in enumerate(floor_dists):
                    origin_frame = int(ani_times[i] * bpy.context.scene.render.fps)
                    frame = origin_frame + context.scene.bc_animation_delay
                    utils.set_keyframe(obj, 'location', frame, (dist + delta, 0, 0))
                for curve in obj.animation_data.action.fcurves:
                    if curve.data_path == 'location':
                        for kf in curve.keyframe_points:
                            kf.interpolation = 'LINEAR'
            obj_bounce_ind += len(note_paths)
        return {'FINISHED'}
